@@ -141,6 +141,18 @@ static vps_config config[] = {
 {"IOPRIO",	NULL, PARAM_IOPRIO},
 {"BOOTORDER",	NULL, PARAM_BOOTORDER},
 
+/* These ones are known parameters for global config file,
+ * it's just vzctl is not interested in those
+ */
+{"VIRTUOZZO",		NULL, PARAM_IGNORED},
+{"VE0CPUUNITS",		NULL, PARAM_IGNORED},
+{"VZFASTBOOT",		NULL, PARAM_IGNORED},
+{"NEIGHBOUR_DEVS",	NULL, PARAM_IGNORED},
+{"ERROR_ON_ARPFAIL",	NULL, PARAM_IGNORED},
+{"VZWDOG",		NULL, PARAM_IGNORED},
+{"IPTABLES_MODULES",	NULL, PARAM_IGNORED},
+{"IP6TABLES",		NULL, PARAM_IGNORED},
+
 {NULL,		NULL, -1}
 };
 
@@ -1018,6 +1030,20 @@ static int parse_dev(vps_param *vps_p, char *val)
 	return 0;
 }
 
+static const char* devperm2str(unsigned long perms)
+{
+	static char mask[4];
+	int i=0;
+	if (perms & S_IROTH)
+		mask[i++] = 'r';
+	if (perms & S_IWOTH)
+		mask[i++] = 'w';
+	if (perms & S_IXGRP)
+		mask[i++] = 'q';
+	mask[i] = 0;
+	return mask;
+}
+
 static int store_dev(vps_param *old_p, vps_param *vps_p, vps_config *conf,
 	list_head_t *conf_h)
 {
@@ -1034,8 +1060,7 @@ static int store_dev(vps_param *old_p, vps_param *vps_p, vps_config *conf,
 	sp = buf;
 	ep = buf + sizeof(buf) - 1;
 	list_for_each(res, &dev->dev, list) {
-		int major, minor, i = 0;
-		char mask[3];
+		int major, minor;
 
 		/* Devices with names (--devnodes) are handled by
 		 * store_devnodes(), so skip those here */
@@ -1045,18 +1070,14 @@ static int store_dev(vps_param *old_p, vps_param *vps_p, vps_config *conf,
 			sp += snprintf(buf, sizeof(buf), "%s=\"", conf->name);
 		major = major(res->dev);
 		minor = minor(res->dev);
-		if (res->mask & S_IROTH)
-			mask[i++] = 'r';
-		if (res->mask & S_IWOTH)
-			mask[i++] = 'w';
-		mask[i] = 0;
 		if (res->use_major) {
 			r = snprintf(sp, ep - sp,"%c:%d:all:%s ",
-				S_ISBLK(res->type) ? 'b' : 'c', major, mask);
+				S_ISBLK(res->type) ? 'b' : 'c', major,
+				devperm2str(res->mask));
 		} else {
 			r = snprintf(sp, ep - sp,"%c:%d:%d:%s ",
 				S_ISBLK(res->type) ? 'b' : 'c', major, minor,
-				mask);
+				devperm2str(res->mask));
 		}
 		sp += r;
 		if ((r < 0) || (sp >= ep))
@@ -1142,9 +1163,8 @@ static int store_devnodes(vps_param *old_p, vps_param *vps_p, vps_config *conf,
 	char buf[STR_SIZE];
 	dev_param *dev = &vps_p->res.dev;
 	dev_res *res;
-	int r, i;
+	int r;
 	char *sp, *ep;
-	char mask[3];
 
 	if (conf->id != PARAM_DEVNODES)
 		return 0;
@@ -1154,19 +1174,14 @@ static int store_devnodes(vps_param *old_p, vps_param *vps_p, vps_config *conf,
 	*sp = 0;
 	ep = buf + sizeof(buf) - 1;
 	list_for_each (res, &dev->dev, list) {
-		i = 0;
 		/* Devices with no names (--devices) are handled by
 		 * store_dev(), so skip those here */
 		if (!res->name)
 			continue;
 		if (sp == buf)
 			sp += snprintf(buf, sizeof(buf), "%s=\"", conf->name);
-		if (res->mask & S_IROTH)
-			mask[i++] = 'r';
-		if (res->mask & S_IWOTH)
-			mask[i++] = 'w';
-		mask[i] = 0;
-		r = snprintf(sp, ep - sp,"%s:%s ", res->name, mask);
+		r = snprintf(sp, ep - sp,"%s:%s ", res->name,
+			devperm2str(res->mask));
 		sp += r;
 		if ((r < 0) || (sp >= ep))
 			break;
@@ -1671,8 +1686,8 @@ static int parse_netif_str_cmd(envid_t veid, const char *str, veth_dev *dev)
 		}
 		err = parse_hwaddr(tmp, dev->dev_addr_ve);
 		if (err) {
-		  logger(-1, 0, "Invalid container MAC address format");
-		  return ERR_INVAL;
+			logger(-1, 0, "Invalid container MAC address format");
+			return ERR_INVAL;
 		}
 	} else {
 		generate_mac(veid, dev->dev_name_ve, dev->dev_addr_ve);
@@ -1795,7 +1810,7 @@ static int parse(envid_t veid, vps_param *vps_p, char *val, int id)
 	if (!_page_size) {
 		_page_size = get_pagesize();
 		if (_page_size < 0)
-			return -1;
+			return ERR_OTHER;
 	}
 	switch (id) {
 	case PARAM_CONFIG:
@@ -1860,17 +1875,17 @@ static int parse(envid_t veid, vps_param *vps_p, char *val, int id)
 		break;
 	case PARAM_LOGLEVEL:
 		if (parse_int(val, &int_id))
-			break;
+			return ERR_INVAL;
 		vps_p->log.level = int_id;
 		break;
 	case PARAM_VERBOSE:
 		if (vps_p->log.verbose != NULL)
-			break;
+			return ERR_DUP;
 		if (parse_int(val, &int_id))
-			break;
+			return ERR_INVAL;
 		vps_p->log.verbose = malloc(sizeof(*vps_p->log.verbose));
 		if (vps_p->log.verbose == NULL)
-			return VZ_RESOURCE_ERROR;
+			return ERR_NOMEM;
 		*vps_p->log.verbose = int_id;
 		break;
 	case PARAM_IPTABLES:
@@ -1949,15 +1964,15 @@ static int parse(envid_t veid, vps_param *vps_p, char *val, int id)
 		break;
 	case PARAM_DISKSPACE:
 		if (vps_p->res.dq.diskspace != NULL)
-			break;
+			return ERR_DUP;
 		if (parse_dq(&vps_p->res.dq.diskspace, val, 1))
-			ret = ERR_INVAL;
+			return ERR_INVAL;
 		break;
 	case PARAM_DISKINODES:
 		if (vps_p->res.dq.diskinodes != NULL)
-			break;
+			return ERR_DUP;
 		if (parse_dq(&vps_p->res.dq.diskinodes, val, 0))
-			ret = ERR_INVAL;
+			return ERR_INVAL;
 		break;
 	case PARAM_QUOTATIME:
 		ret = conf_parse_ulong(&vps_p->res.dq.exptime, val);
@@ -1993,7 +2008,7 @@ static int parse(envid_t veid, vps_param *vps_p, char *val, int id)
 		break;
 	case PARAM_CPULIMIT:
 		if (vps_p->res.cpu.limit != NULL)
-			break;
+			return ERR_DUP;
 		ret = parse_cpulimit(&vps_p->res.cpu.limit, val);
 		break;
 	case PARAM_VCPUS:
@@ -2011,7 +2026,7 @@ static int parse(envid_t veid, vps_param *vps_p, char *val, int id)
 		break;
 	case PARAM_NETIF_ADD:
 		if (!list_empty(&vps_p->res.veth.dev))
-			break;
+			return ERR_DUP;
 		ret = parse_netif(veid, &vps_p->res.veth, val);
 		break;
 	case PARAM_NETIF_ADD_CMD:
@@ -2046,7 +2061,7 @@ static int parse(envid_t veid, vps_param *vps_p, char *val, int id)
 		break;
 	case PARAM_NAME:
 		if (vps_p->res.name.name != NULL)
-			break;
+			return ERR_DUP;
 		if (check_name(val))
 			return ERR_INVAL;
 		ret = conf_parse_str(&vps_p->res.name.name, val);
@@ -2066,6 +2081,9 @@ static int parse(envid_t veid, vps_param *vps_p, char *val, int id)
 	case PARAM_BOOTORDER:
 		ret = conf_parse_ulong(&vps_p->res.misc.bootorder, val);
 		break;
+	case PARAM_IGNORED:
+		/* Well known but ignored parameter */
+		break;
 	default:
 		logger(10, 0, "Not handled parameter %d %s", id, val);
 		break;
@@ -2139,30 +2157,48 @@ int vps_parse_config(envid_t veid, char *path, vps_param *vps_p,
 			ret = parse(veid, vps_p, rtoken, conf->id);
 		} else if (action != NULL)
 			ret = mod_parse(veid, action, ltoken, -1, rtoken);
-		else
+		else {
+			logger(1, 0, "Warning at %s:%d: unknown parameter "
+				"%s (\"%s\"), ignored",
+				path, line, ltoken, rtoken);
 			continue;
+		}
 		if (!ret) {
 			continue;
 		} else if (ret == ERR_INVAL_SKIP) {
+			/* Warning is printed by parse() */
 			continue;
 		} else if (ret == ERR_LONG_TRUNC) {
-			logger(-1, 0, "Warning: too large value for %s=%s"
-				" was truncated", ltoken, rtoken);
+			logger(-1, 0, "Warning at %s:%d: too large value "
+				"for %s (\"%s\"), truncated",
+				path, line, ltoken, rtoken);
 		} else if (ret == ERR_DUP) {
-			logger(-1, 0, "Warning: dup for %s=%s in line %d"
-				" is ignored", ltoken, rtoken, line);
+			logger(-1, 0, "Warning at %s:%d: duplicate "
+				"for %s (\"%s\"), ignored",
+				path, line, ltoken, rtoken);
 		} else if (ret == ERR_INVAL) {
-			logger(-1, 0, "Invalid value for %s=%s, skipped",
-				ltoken, rtoken);
+			logger(-1, 0, "Warning at %s:%d: invalid value "
+				"for %s (\"%s\"), skipped",
+				path, line, ltoken, rtoken);
 		} else if (ret == ERR_UNK) {
-			logger(-1, 0, "Unknown parameter %s, skipped", ltoken);
+			logger(1, 0, "Warning at %s:%d: unknown parameter "
+				"%s (\"%s\"), ignored",
+				path, line, ltoken, rtoken);
 		} else if (ret == ERR_NOMEM) {
-			logger(-1, 0, "Not enough memory");
+			logger(-1, ENOMEM, "Error while parsing %s:%d",
+				path, line);
 			err = VZ_RESOURCE_ERROR;
 			break;
+		} else if (ret == ERR_OTHER) {
+			logger(-1, 0, "System error while parsing %s:%d",
+				path, line);
+			err = VZ_SYSTEM_ERROR;
+			break;
 		} else {
-			logger(-1, 0, "Unknown exit code %d on parse %s",
-				ret, ltoken);
+			logger(-1, 0, "Internal error at %s:%d: "
+				"bad return value %d from parse(), "
+				"parameter %s (\"%s\")", path, line,
+				ret, ltoken, rtoken);
 		}
 	}
 	fclose(fp);
@@ -2204,7 +2240,7 @@ static int write_conf(char *fname, list_head_t *head)
 	if (file == NULL) {
 		if (errno != ENOENT) {
 			logger(-1, errno, "Unable to resolve path %s", fname);
-			return 1;
+			return ret;
 		}
 		file = vz_strdup(fname);
 		if (!file)
@@ -2281,7 +2317,7 @@ int vps_save_config(envid_t veid, char *path, vps_param *new_p,
 {
 	vps_param *tmp_old_p = NULL;
 	list_head_t conf, new_conf;
-	int ret, n;
+	int ret = VZ_CONFIG_SAVE_ERROR;
 
 	list_head_init(&conf);
 	list_head_init(&new_conf);
@@ -2290,15 +2326,27 @@ int vps_save_config(envid_t veid, char *path, vps_param *new_p,
 		vps_parse_config(veid, path, tmp_old_p, action);
 		old_p = tmp_old_p;
 	}
-	if ((ret = read_conf(path, &conf)))
-		return ret;
-	n = store(old_p, new_p, &new_conf);
+	if (read_conf(path, &conf) != 0)
+		goto err_read;
+	store(old_p, new_p, &new_conf);
 	if (action != NULL)
 		mod_save_config(action, &new_conf);
-	if ((ret = vps_merge_conf(&conf, &new_conf)) > 0)
-		ret = write_conf(path, &conf);
+	if (vps_merge_conf(&conf, &new_conf) == 0)
+	{
+		/* Nothing to save */
+		logger(0, 0, "No changes in CT configuration, not saving");
+		ret = 0;
+		goto out;
+	}
+
+	ret = write_conf(path, &conf);
+	if (ret == 0)
+		logger(0, 0, "Saved parameters "
+				"for CT %d", veid);
+out:
 	free_str_param(&conf);
 	free_str_param(&new_conf);
+err_read:
 	free_vps_param(tmp_old_p);
 
 	return ret;
