@@ -63,7 +63,7 @@ static int set_personality(unsigned long mask)
 	per = personality(0xffffffff) | mask;
 	logger(3, 0, "Set personality %#10.8lx", per);
 	if (personality(per) == -1) {
-		logger(2, errno, "Unable to set personality PER_LINUX32");
+		logger(-1, errno, "Unable to set personality PER_LINUX32");
 		return  -1;
 	}
 	return 0;
@@ -297,6 +297,18 @@ static int vz_env_configure(int fd, envid_t veid, const char *osrelease)
 	return ret;
 }
 
+static int configure_sysctl()
+{
+	int fd;
+
+	fd = open("/proc/sys/net/ipv6/conf/all/forwarding", O_WRONLY);
+	if (fd == -1)
+		return -1;
+	write(fd, "0", 1);
+	close(fd);
+	return 0;
+}
+
 static int _env_create(vps_handler *h, envid_t veid, int wait_p, int err_p,
 	void *data)
 {
@@ -399,7 +411,7 @@ try:
 		make_dir("/var/lib/nfs/rpc_pipefs", 1);
 		mount("sunrpc", "/var/lib/nfs/rpc_pipefs", "rpc_pipefs", 0, 0);
 	}
-
+	configure_sysctl();
 	if (res->dq.ugidlimit != NULL)
 		mk_quota_link();
 
@@ -619,11 +631,13 @@ int vps_start_custom(vps_handler *h, envid_t veid, vps_param *param,
 		logger(-1, errno, "Can not create pipe");
 		return VZ_RESOURCE_ERROR;
 	}
-	/* old_wait_p is needed for backward compatibily with old kernels.
-	 * Now we use wait_p for this purpose. If old_wait_p is closed without
-	 * writing any data, it's "OK to go" signal and if data are received
-	 * from old_wait_p it's "no go" signal". It doesn't work if vzctl
-	 * segfaults, because in this case the decriptor will be closed without
+	/* old_wait_p is needed for backward compatibility with older kernels,
+	 * while for recent ones (that support CPT_SET_LOCKFD2) we use wait_p.
+	 *
+	 * If old_wait_p is closed without writing any data, it's "OK to go"
+	 * signal, and if data are received from old_wait_p it's "no go"
+	 * signal". Note that such thing doesn't work if vzctl segfaults,
+	 * because in this case the descriptor will be closed without
 	 * sending data.
 	 */
 	if (pipe(old_wait_p) < 0) {
